UNITED_ARAB_EMIRATES_DIRHAM = "AED"
AFGHAN_AFGHANI = "AFN"
ALBANIAN_LEK = "ALL"
ARMENIAN_DRAM = "AMD"
ARGENTINE_PESO = "ARS"
AUSTRALIAN_DOLLAR = "AUD"
AZERBAIJANI_MANAT = "AZN"
BOSNIA_AND_HERZEGOVINA_CONVERTIBLE_MARK = "BAM"
BANGLADESHI_TAKA = "BDT"
BULGARIAN_LEV = "BGN"
BRUNEI_DOLLAR = "BND"
BOLIVIAN_BOLIVIANO = "BOB"
BRAZILIAN_REAL = "BRL"
CANADIAN_DOLLAR = "CAD"
SWISS_FRANC = "CHF"
CHILEAN_PESO = "CLP"
CHINESE_RENMINBI_YUAN = "CNY"
COLOMBIAN_PESO = "COP"
COSTA_RICAN_COLON = "CRC"
CZECH_KORUNA = "CZK"
DANISH_KRONE = "DKK"
DOMINICAN_PESO = "DOP"
ALGERIAN_DINAR = "DZD"
EGYPTIAN_POUND = "EGP"
EURO = "EUR"
BRITISH_POUND = "GBP"
GEORGIAN_LARI = "GEL"
GUATEMALAN_QUETZAL = "GTQ"
HONG_KONG_DOLLAR = "HKD"
HONDURAN_LEMPIRA = "HNL"
CROATIAN_KUNA = "HRK"
HUNGARIAN_FORINT = "HUF"
INDONESIAN_RUPIAH = "IDR"
ISRAELI_NEW_SHEQEL = "ILS"
INDIAN_RUPEE = "INR"
ICELANDIC_KRONA = "ISK"
JAMAICAN_DOLLAR = "JMD"
JAPANESE_YEN = "JPY"
KENYAN_SHILLING = "KES"
KYRGYZSTANI_SOM = "KGS"
SOUTH_KOREAN_WON = "KRW"
KAZAKHSTANI_TENGE = "KZT"
LEBANESE_POUND = "LBP"
SRI_LANKAN_RUPEE = "LKR"
MOROCCAN_DIRHAM = "MAD"
MOLDOVAN_LEU = "MDL"
MONGOLIAN_TOGROG = "MNT"
MAURITIAN_RUPEE = "MUR"
MALDIVIAN_RUFIYAA = "MVR"
MEXICAN_PESO = "MXN"
MALAYSIAN_RINGGIT = "MYR"
MOZAMBICAN_METICAL = "MZN"
NIGERIAN_NAIRA = "NGN"
NICARAGUAN_CORDOBA = "NIO"
NORWEGIAN_KRONE = "NOK"
NEPALESE_RUPEE = "NPR"
NEW_ZEALAND_DOLLAR = "NZD"
PANAMANIAN_BALBOA = "PAB"
PERUVIAN_NUEVO_SOL = "PEN"
PHILIPPINE_PESO = "PHP"
PAKISTANI_RUPEE = "PKR"
POLISH_ZLOTY = "PLN"
PARAGUAYAN_GUARANI = "PYG"
QATARI_RIYAL = "QAR"
ROMANIAN_LEU = "RON"
SERBIAN_DINAR = "RSD"
RUSSIAN_RUBLE = "RUB"
SAUDI_RIYAL = "SAR"
SWEDISH_KRONA = "SEK"
SINGAPORE_DOLLAR = "SGD"
THAI_BAHT = "THB"
TAJIKISTANI_SOMONI = "TJS"
TURKISH_LIRA = "TRY"
TRINIDAD_AND_TOBAGO_DOLLAR = "TTD"
NEW_TAIWAN_DOLLAR = "TWD"
TANZANIAN_SHILLING = "TZS"
UKRAINIAN_HRYVNIA = "UAH"
UGANDAN_SHILLING = "UGX"
UNITED_STATES_DOLLAR = "USD"
URUGUAYAN_PESO = "UYU"
UZBEKISTANI_SOM = "UZS"
VIETNAMESE_DONG = "VND"
YEMENI_RIAL = "YER"
SOUTH_AFRICAN_RAND = "ZAR"
